@@ -1,4 +1,4 @@
-/* 
+/*
  *  dv.c
  *
  *     Copyright (C) Charles 'Buck' Krasic - November 2000
@@ -7,17 +7,17 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
@@ -94,7 +94,7 @@ dv_decoder_new(int add_ntsc_setup, int clamp_luma, int clamp_chroma) {
   result = (dv_decoder_t *)calloc(1,sizeof(dv_decoder_t));
   if(!result) goto no_mem;
   
-  result->add_ntsc_setup = add_ntsc_setup;
+  result->add_ntsc_setup = FALSE;
   result->clamp_luma = clamp_luma;
   result->clamp_chroma = clamp_chroma;
   dv_init(clamp_luma, clamp_chroma);
@@ -123,15 +123,6 @@ dv_decoder_new(int add_ntsc_setup, int clamp_luma, int clamp_chroma) {
     arg: &result->arg_video_system,
   }; /* system */
 
-  result->option_table[DV_DECODER_OPT_NTSCSETUP] = (struct poptOption){
-    longName:  "ntsc-setup", 
-    arg:       &result->add_ntsc_setup,
-    argInfo: POPT_ARG_INT, 
-    argDescrip: "(0|1)",
-    descrip:   "add 7.5 IRE setup to NTSC only: "
-    " 0=off, 1=on [default]"
-  }; /* ntsc-setup */
-
   result->option_table[DV_DECODER_OPT_VIDEO_INCLUDE] = (struct poptOption) {
     argInfo: POPT_ARG_INCLUDE_TABLE,
     descrip: "Video decode options",
@@ -170,6 +161,7 @@ dv_decoder_free( dv_decoder_t *decoder)
 	if (decoder != NULL) {
 		if (decoder->audio != NULL) free(decoder->audio);
 		if (decoder->video != NULL) free(decoder->video);
+		if (decoder->bs != NULL) free(decoder->bs);
 		free(decoder);
 	}
 } /* dv_decoder_free */
@@ -182,6 +174,8 @@ dv_init(int clamp_luma, int clamp_chroma) {
 #if ARCH_X86
   dv_use_mmx = mmx_ok(); 
 #endif
+
+  /* decoder */
   _dv_weight_init();
   _dv_dct_init();
   dv_dct_248_init();
@@ -192,11 +186,13 @@ dv_init(int clamp_luma, int clamp_chroma) {
   dv_rgb_init(clamp_luma, clamp_chroma);
   dv_YUY2_init(clamp_luma, clamp_chroma);
   dv_YV12_init(clamp_luma, clamp_chroma);
+
+  /* encoder */
   _dv_init_vlc_test_lookup();
   _dv_init_vlc_encode_lookup();
   _dv_init_qno_start();
   _dv_prepare_reorder_tables();
-
+  
   done=TRUE;
  init_done:
   return;
@@ -444,7 +440,7 @@ void dv_check_coeff_ranges(dv_macroblock_t *mb) {
 }
 
 void
-dv_decode_full_frame(dv_decoder_t *dv, const uint8_t *buffer, 
+dv_decode_full_frame(dv_decoder_t *dv, const uint8_t *buffer,
 		     dv_color_space_t color_space, uint8_t **pixels, int *pitches) {
 
   static dv_videosegment_t vs;
@@ -454,8 +450,8 @@ dv_decode_full_frame(dv_decoder_t *dv, const uint8_t *buffer,
   unsigned int offset = 0, dif = 0, audio=0;
 
   if(!seg->bs) {
-    seg->bs = _dv_bitstream_init();
-    if(!seg->bs) 
+    dv->bs = seg->bs = _dv_bitstream_init();
+    if(!seg->bs)
       goto no_mem;
   } /* if */
   seg->isPAL = (dv->system == e_dv_system_625_50);
@@ -468,7 +464,7 @@ dv_decode_full_frame(dv_decoder_t *dv, const uint8_t *buffer,
        block contains one compressed macroblock.  DV bit allocation
        for the VLC stage can spill bits between blocks in the same
        video segment.  So parsing needs the whole segment to decode
-       the VLC data 
+       the VLC data
 	*/
     dif += 6;
     audio=0;
@@ -476,13 +472,13 @@ dv_decode_full_frame(dv_decoder_t *dv, const uint8_t *buffer,
     for (v=0;v<27;v++) {
       /* skip audio block - interleaved before every 3rd video segment */
       if(!(v % 3)) {
-	/*dv_dump_aaux_as(buffer+(dif*80), ds, audio); */
-	dif++; 
+        /*dv_dump_aaux_as(buffer+(dif*80), ds, audio); */
+	dif++;
 	audio++;
       } /* if */
       /* stage 1: parse and VLC decode 5 macroblocks that make up a video segment */
       offset = dif * 80;
-      _dv_bitstream_new_buffer(seg->bs, (uint8_t *)buffer + offset, 80*5); 
+      _dv_bitstream_new_buffer(seg->bs, (uint8_t *)buffer + offset, 80*5);
       dv_parse_video_segment(seg, dv->quality);
       /* stage 2: dequant/unweight/iDCT blocks, and place the macroblocks */
       dif+=5;
@@ -548,54 +544,56 @@ dv_frame_has_audio_errors (dv_decoder_t *dv)
 
 /* ---------------------------------------------------------------------------
  */
-int 
+int
 dv_decode_full_audio(dv_decoder_t *dv, const uint8_t *buffer, int16_t **outbufs)
 {
   int ds, dif, audio_dif, result;
   int ch;
 
   dif=0;
-
-  if(!dv_parse_audio_header(dv, buffer)) goto no_audio;
-  dv->audio->block_failure = dv->audio->sample_failure = 0;
-  for (ds=0; ds < dv->num_dif_seqs; ds++) {
+  if (!dv_parse_audio_header (dv, buffer))
+  {
+    goto no_audio;
+  }
+  dv -> audio -> block_failure = dv -> audio -> sample_failure = 0;
+  for (ds = 0; ds < dv -> num_dif_seqs; ds++) {
     dif += 6;
-    for(audio_dif=0; audio_dif<9; audio_dif++) {
-      if((result = dv_decode_audio_block(dv->audio, buffer+(dif *80), ds, audio_dif, outbufs))) goto fail;
-      dif+=16;
+    for (audio_dif = 0; audio_dif < 9; audio_dif++) {
+      if ((result = dv_decode_audio_block (dv -> audio,
+                                           buffer + (dif * 80),
+                                           ds,
+                                           audio_dif,
+                                           outbufs)))
+      {
+        goto fail;
+      }
+      dif += 16;
     } /* for  */
   } /* for */
 
-  if (dv -> audio -> block_failure) {
+  if (dv -> audio -> sample_failure) {
     if (dv -> audio -> error_log) {
       fprintf (dv -> audio -> error_log,
-               "# audio block failure for %d blocks = %d samples of %d\n",
+               "# audio block/sample failure for %d blocks, %d samples of %d\n",
                dv -> audio -> block_failure,
                dv -> audio -> sample_failure,
-               dv -> audio -> samples_this_frame);
+               dv -> audio -> raw_samples_this_frame [0]); /* TODO: second channel */
     }
     dv_audio_correct_errors (dv -> audio, outbufs);
   }
-  if (!dv->audio->block_failure && dv->audio->sample_failure) {
-    if (dv -> audio -> error_log) {
-      fprintf (dv -> audio -> error_log,
-               "# sample failure without block failure: "
-                 "report this to libdv at SF!!\n");
-    }
-  }
 
-  if(dv->audio->emphasis) {
-    for(ch=0; ch< dv->audio->num_channels; ch++) {
-      dv_audio_deemphasis(dv->audio, outbufs[ch]);
-    } /* for  */
-  } /* if */
+  dv_audio_deemphasis (dv -> audio, outbufs);
+
+  dv_audio_mix4ch (dv -> audio, outbufs);
 
   return(TRUE);
-  
+
  fail:
+fprintf (stderr, "# decode failure \n");
  no_audio:
+fprintf (stderr, "# no audio\n");
   return(FALSE);
-  
+
 } /* dv_decode_full_audio */
 
 /* ---------------------------------------------------------------------------
@@ -619,6 +617,7 @@ dv_report_video_error (dv_decoder_t *dv, uint8_t *data)
      */
     if ((data [i] & 0xe0) == 0x80) {
       error_code = data [i + 3] >> 4;
+#if 0
       if (error_code) {
         dv_get_timestamp (dv, err_msg1);
         dv_get_recording_datetime (dv, err_msg2);
@@ -633,10 +632,30 @@ dv_report_video_error (dv_decoder_t *dv, uint8_t *data)
                  data [i + 2],
                  data [i + 3]);
       }
+#endif
     }
   }
 }
 
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_is_PAL (dv_decoder_t *dv)
+{
+  return dv -> system == e_dv_system_625_50;
+}
+
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_set_quality (dv_decoder_t *dv, int q)
+{
+    int old_q = dv -> quality;
+
+  dv -> quality = q;
+  return old_q;
+}
+
 /* ---------------------------------------------------------------------------
  * query functions based upon VAUX data
  */
@@ -653,6 +672,18 @@ dv_get_vaux_pack (dv_decoder_t *dv, uint8_t pack_id, uint8_t *data)
   return 0;
 } /* dv_get_vaux_pack */
 
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_get_ssyb_pack (dv_decoder_t *dv, uint8_t pack_id, uint8_t *data)
+{
+  uint8_t  id;
+  if ((id = dv -> ssyb_pack [pack_id]) == 0xff) 
+    return -1;
+  memcpy (data, dv -> ssyb_data [id], 4);
+  return 0;
+} /* dv_get_vaux_pack */
+
 /* ---------------------------------------------------------------------------
  */
 int
@@ -733,6 +764,22 @@ dv_frame_changed (dv_decoder_t *dv)
   return -1;
 }
 
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_is_progressive (dv_decoder_t *dv)
+{
+  uint8_t  id;
+
+  if ((id = dv -> vaux_pack [0x61]) != 0xff) {
+    if (dv -> vaux_data [id] [2] & 0x08) {
+      return 0;
+    }
+    return 1;
+  }
+  return -1;
+}
+
 /* ---------------------------------------------------------------------------
  * query functions based upon SSYB data
  * decoding code is taken from kino
@@ -816,6 +863,28 @@ dv_get_recording_datetime (dv_decoder_t *dv, char *dtptr)
               (dv -> ssyb_data [id2] [1] & 0x0f));
     return 1;
   }
+  
+  /* it may also be found in vaux for some... */
+  if ((id1 = dv -> vaux_pack [0x62]) != 0xff &&
+      (id2 = dv -> vaux_pack [0x63]) != 0xff) {
+    year = dv -> vaux_data [id1] [3];
+    year = (year & 0x0f) + 10 * ((year >> 4) & 0x0f);
+    year += (year < 25) ? 2000 : 1900;
+    sprintf (dtptr,
+             "%04d-%02d-%02d %02d:%02d:%02d",
+             year,
+             ((dv -> vaux_data [id1] [2] >> 4) & 0x01) * 10 +
+               (dv -> vaux_data [id1] [2] & 0x0f),
+             ((dv -> vaux_data [id1] [1] >> 4) & 0x03) * 10 +
+               (dv -> vaux_data [id1] [1] & 0x0f),
+             ((dv -> vaux_data [id2] [3] >> 4) & 0x03) * 10 +
+              (dv -> vaux_data [id2] [3] & 0x0f),
+             ((dv -> vaux_data [id2] [2] >> 4) & 0x07) * 10 +
+              (dv -> vaux_data [id2] [2] & 0x0f),
+             ((dv -> vaux_data [id2] [1] >> 4) & 0x07) * 10 +
+              (dv -> vaux_data [id2] [1] & 0x0f));
+    return 1;
+  }
   strcpy (dtptr, "0000-00-00 00:00:00");
   return 0;
 }
@@ -852,6 +921,33 @@ dv_get_recording_datetime_tm (dv_decoder_t *dv, struct tm *rec_dt)
 
     return 1;
   }
+  
+  /* it may also be found in vaux for some... */
+  if ((id1 = dv -> vaux_pack [0x62]) != 0xff &&
+      (id2 = dv -> vaux_pack [0x63]) != 0xff) {
+    year = dv -> vaux_data [id1] [3];
+    year = (year & 0x0f) + 10 * ((year >> 4) & 0x0f);
+    year += (year < 25) ? 2000 : 1900;
+    rec_dt -> tm_isdst = rec_dt -> tm_yday = rec_dt -> tm_wday = -1;
+    rec_dt -> tm_year = year - 1900;
+    rec_dt -> tm_mon = ((dv -> vaux_data [id1] [2] >> 4) & 0x01) * 10 +
+                        (dv -> vaux_data [id1] [2] & 0x0f) - 1;
+    rec_dt -> tm_mday = ((dv -> vaux_data [id1] [1] >> 4) & 0x03) * 10 +
+                         (dv -> vaux_data [id1] [1] & 0x0f);
+    rec_dt -> tm_hour = ((dv -> vaux_data [id2] [3] >> 4) & 0x03) * 10 +
+                         (dv -> vaux_data [id2] [3] & 0x0f);
+    rec_dt -> tm_min  = ((dv -> vaux_data [id2] [2] >> 4) & 0x07) * 10 +
+                         (dv -> vaux_data [id2] [2] & 0x0f);
+    rec_dt -> tm_sec  = ((dv -> vaux_data [id2] [1] >> 4) & 0x07) * 10 +
+                         (dv -> vaux_data [id2] [1] & 0x0f);
+    /* -----------------------------------------------------------------------
+     * sanity check
+     */
+    if (mktime (rec_dt) == -1)
+      return 0;
+
+    return 1;
+  }
   return 0;
 }
 
@@ -861,11 +957,12 @@ FILE *
 dv_set_error_log (dv_decoder_t *dv, FILE *log)
 {
     FILE *old_log;
-    
+
   old_log = dv -> audio -> error_log;
   dv -> audio -> error_log =
     dv -> video -> error_log =
       log;
   return old_log;
 }
+
 /*@}*/
