@@ -7,17 +7,17 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
@@ -809,6 +809,14 @@ static void pgm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 
 	emms();
 #endif
+	{
+		register int i, j;
+		register short min = (16 - 128) << DCT_YUV_PRECISION;
+		register short max = (235 - 128) << DCT_YUV_PRECISION;
+		for (i = 0; i < 64; i++)
+			for (j = 0; j < 4; j++)
+				bl[j].coeffs[i] = CLAMP(bl[j].coeffs[i], min, max);
+	}
 }
 
 
